'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _d3Selection = require('d3-selection');

var _d3Scale = require('d3-scale');

var _d3Axis = require('d3-axis');

var _d3Shape = require('d3-shape');

require('d3-transition');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var defaults = {

  width: 800,

  height: 400,

  margin: {
    top: 15,
    right: 10,
    bottom: 35,
    left: 60
  },

  tickSize: 5,

  yTicks: 5,

  curve: _d3Shape.curveLinear,

  tip: {
    width: 250,
    height: 85,
    margin: 25
  }
};

/**
 * Line chart.
 */

var LineChart = function () {
  function LineChart(config) {
    _classCallCheck(this, LineChart);

    Object.assign(this, defaults, config);
    this.init();
  }

  _createClass(LineChart, [{
    key: 'init',
    value: function init() {
      var _this = this;

      var target = this.target;
      var width = this.width;
      var height = this.height;
      var margin = this.margin;
      var curve = this.curve;
      var tickSize = this.tickSize;
      var yTicks = this.yTicks;

      var w = this.w = width - margin.left - margin.right;
      var h = this.h = height - margin.top - margin.bottom;

      this.color = (0, _d3Scale.scaleOrdinal)(_d3Scale.schemeCategory10);

      var wrapper = (0, _d3Selection.select)(target).append('div').style('position', 'relative');

      this.chart = wrapper.append('svg').attr('width', width).attr('height', height).append('g').attr('transform', 'translate(' + margin.left + ', ' + margin.top + ')');

      this.x = (0, _d3Scale.scaleLinear)().range([0, w]);

      this.y = (0, _d3Scale.scaleLinear)().range([h, 0]);

      this.xAxis = (0, _d3Axis.axisBottom)(this.x).ticks(0);

      this.yAxis = (0, _d3Axis.axisLeft)(this.y).ticks(yTicks).tickPadding(8).tickSize(tickSize);

      this.chart.append('g').attr('class', 'y grid');

      this.chart.append('g').attr('class', 'x axis').attr('transform', 'translate(0, ' + h + ')');

      this.chart.append('g').attr('class', 'y axis');

      this.line = (0, _d3Shape.line)().x(function (d, i) {
        return _this.x(i);
      }).y(function (d) {
        return _this.y(d);
      }).curve(curve);

      this.area = (0, _d3Shape.area)().x(function (d, i) {
        return _this.x(i);
      }).y0(h).y1(function (d) {
        return _this.y(d);
      });

      var tip = wrapper.append('div').attr('class', 'linechart tip');

      tip.append('div').attr('class', 'tip title');

      var teamA = tip.append('div').attr('class', 'tip team').style('color', function () {
        return _this.color(0);
      });

      teamA.append('span').attr('class', 'tip team teamA name');

      teamA.append('span').attr('class', 'tip team teamA score');

      var teamB = tip.append('div').attr('class', 'tip team').style('color', function () {
        return _this.color(1);
      });

      teamB.append('span').attr('class', 'tip team teamB name');

      teamB.append('span').attr('class', 'tip team teamB score');
    }

    /**
     * Render axis.
     */

  }, {
    key: 'renderAxis',
    value: function renderAxis(data) {
      var chart = this.chart;
      var x = this.x;
      var y = this.y;
      var xAxis = this.xAxis;
      var yAxis = this.yAxis;

      var count = data.length - 1;
      x.domain([0, count]);
      y.domain([0, Math.max(data[count].scoreTeamA, data[count].scoreTeamB)]);
      var c = chart.transition();
      c.select('.x.axis').call(xAxis);
      c.select('.y.axis').call(yAxis);
    }

    /**
     * Render dots.
     */

  }, {
    key: 'renderDots',
    value: function renderDots(data) {
      var group = [data.map(function (d) {
        return d.scoreTeamA;
      }), data.map(function (d) {
        return d.scoreTeamB;
      })];
      this._drawCircles(group[0], 0, 'a');
      this._drawCircles(group[1], 1, 'b');
    }

    /**
     * Draw circles for single team.
     */

  }, {
    key: '_drawCircles',
    value: function _drawCircles(data, i, side) {
      var _this2 = this;

      var s = this.chart.selectAll('.dot.' + side).data(data);

      // enter
      s.enter().append('circle').attr('class', 'dot ' + side).attr('cx', function (d, i) {
        return _this2.x(i);
      }).attr('cy', function (d) {
        return _this2.y(d);
      }).transition().attr('r', 5).style('fill', function () {
        return _this2.color(i);
      }).style('stroke', '#fff').style('stroke-width', '2px');

      // transition
      s.transition().attr('cx', function (d, i) {
        return _this2.x(i);
      }).attr('cy', function (d) {
        return _this2.y(d);
      });

      // exit
      s.exit().transition().remove();
    }

    /**
     * Render overlay.
     */

  }, {
    key: 'renderOverlay',
    value: function renderOverlay(data) {
      var _this3 = this;

      // const group = [data.map(d => d.scoreTeamA), data.map(d => d.scoreTeamB)]
      this.chart.append('rect').attr('class', 'overlay').attr('width', this.w).attr('height', this.h).style('fill', 'transparent').on('mousemove', function (event) {
        var m = (0, _d3Selection.mouse)(_this3.chart.node());
        var x = _this3.x.invert(m[0]);
        var i = Math.round(x);

        // highlight current dots for team a and team b
        _this3.chart.selectAll('.dot.a').style('stroke', function (d, j) {
          return i === j ? _this3.color(0) : '#fff';
        });

        _this3.chart.selectAll('.dot.b').style('stroke', function (d, j) {
          return i === j ? _this3.color(1) : '#fff';
        });

        var scoreTeamA = data[i].scoreTeamA;
        var scoreTeamB = data[i].scoreTeamB;
        var max = scoreTeamA > scoreTeamB ? scoreTeamA : scoreTeamB;
        var min = scoreTeamA < scoreTeamB ? scoreTeamA : scoreTeamB;

        var left = _this3.x(i) + _this3.margin.left;
        var top = _this3.y(max) + _this3.margin.top;
        var bottom = _this3.y(min) + _this3.margin.top;
        var sel = (0, _d3Selection.select)('.linechart.tip');
        // check left edge
        if (left - _this3.tip.width / 2 - _this3.margin.left <= 0) {
          // left edge
          var xFixed = _this3.x.invert(_this3.tip.width / 2);
          sel.style('left', _this3.x(Math.ceil(xFixed)) + _this3.margin.left - _this3.tip.width / 2 + 'px');
        } else if (left >= _this3.w - _this3.tip.width / 2) {
          // right edge
          var _xFixed = _this3.x.invert(_this3.w - _this3.tip.width / 2);
          sel.style('left', _this3.x(Math.floor(_xFixed)) + _this3.margin.left - _this3.tip.width / 2 + 'px');
        } else {
          // default
          sel.style('left', left - _this3.tip.width / 2 + 'px');
        }

        if (top - _this3.tip.height - _this3.tip.margin > 0) {
          // default
          sel.style('top', top - _this3.tip.height - _this3.tip.margin + 'px');
        } else {
          // check top edge
          sel.style('top', bottom + _this3.tip.margin + 'px');
        }

        // update title
        (0, _d3Selection.select)('.tip.title').text(data[i].description + ' by ' + data[i].firstName + ' ' + data[i].lastName);

        // update score
        (0, _d3Selection.select)('.tip.teamA.score').text('' + data[i].scoreTeamA);

        (0, _d3Selection.select)('.tip.teamB.score').text('' + data[i].scoreTeamB);

        // hide tip at origin
        var opacity = 1;
        if (i === 0) {
          opacity = 0;
        }
        sel.style('opacity', opacity);
      }).on('mouseleave', function () {
        // deselect dots
        _this3.chart.selectAll('.dot').style('stroke', '#fff');
        // hide tooltip
        (0, _d3Selection.select)('.linechart.tip').transition().style('opacity', 0);
      });
    }

    /**
     * Render area.
     */

  }, {
    key: 'renderArea',
    value: function renderArea(data) {
      var _this4 = this;

      var group = [data.map(function (d) {
        return d.scoreTeamA;
      }), data.map(function (d) {
        return d.scoreTeamB;
      })];

      var s = this.chart.selectAll('.area').data(group);

      // enter
      s.enter().append('path').attr('class', 'area').style('opacity', 0.5).style('fill', function (d, i) {
        return _this4.color(i);
      }).transition().attr('d', function (d, i) {
        return _this4.area(group[i]);
      });

      // transition
      s.transition().attr('d', function (d, i) {
        return _this4.area(group[i]);
      });

      // exit
      s.exit().remove();
    }

    /**
     * Render line.
     */

  }, {
    key: 'renderLine',
    value: function renderLine(data) {
      var _this5 = this;

      var group = [data.map(function (d) {
        return d.scoreTeamA;
      }), data.map(function (d) {
        return d.scoreTeamB;
      })];

      var s = this.chart.selectAll('.line').data(group);

      // enter
      s.enter().append('path').attr('class', 'line').transition().attr('d', function (d, i) {
        return _this5.line(group[i]);
      }).style('stroke', function (d, i) {
        return _this5.color(i);
      });

      // transition
      s.transition().attr('d', function (d, i) {
        return _this5.line(group[i]);
      });

      // exit
      s.exit().transition().remove();
    }

    /**
     * Render grid.
     */

  }, {
    key: 'renderGrid',
    value: function renderGrid(data) {
      var yTicks = this.yTicks;

      this.chart.selectAll('.grid.y').transition().call((0, _d3Axis.axisLeft)(this.y).ticks(yTicks).tickSizeInner(-this.w).tickSizeOuter(0).tickFormat(''));
    }

    /**
     * Set team names in tip.
     */

  }, {
    key: 'setTeams',
    value: function setTeams(sideA, sideB) {
      (0, _d3Selection.select)('.tip.team.teamA.name').text(sideA);
      (0, _d3Selection.select)('.tip.team.teamB.name').text(sideB);
    }

    /**
     * Render.
     */

  }, {
    key: 'render',
    value: function render(data) {
      // render axis first because it sets the x and y domains
      this.renderAxis(data);
      this.renderGrid(data);
      this.renderArea(data);
      this.renderLine(data);
      this.renderDots(data);
      this.renderOverlay(data);
    }

    /**
     * Update
     */

  }, {
    key: 'update',
    value: function update(data) {
      this.render(data);
    }
  }]);

  return LineChart;
}();

exports.default = LineChart;